@@ -3,7 +3,7 @@
  * make-iso.c: code to generate iso files
  *
  * Copyright (C) 2002-2004 Red Hat, Inc.
- * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2005-2006 William Jon McCann <mccann@jhu.edu>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -623,6 +623,45 @@ mkisofs_output_error (struct mkisofs_output *mkisofs_output,
 	mkisofs_output->result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 }
 
+/* Originally eel_make_valid_utf8 */
+static char *
+ncb_make_valid_utf8 (const char *name)
+{
+	GString    *string;
+	const char *remainder, *invalid;
+	int         remaining_bytes, valid_bytes;
+
+	string = NULL;
+	remainder = name;
+	remaining_bytes = strlen (name);
+
+	while (remaining_bytes != 0) {
+		if (g_utf8_validate (remainder, remaining_bytes, &invalid)) {
+			break;
+		}
+		valid_bytes = invalid - remainder;
+
+		if (string == NULL) {
+			string = g_string_sized_new (remaining_bytes);
+		}
+		g_string_append_len (string, remainder, valid_bytes);
+		g_string_append_c (string, '?');
+
+		remaining_bytes -= valid_bytes + 1;
+		remainder = invalid + 1;
+	}
+
+	if (string == NULL) {
+		return g_strdup (name);
+	}
+
+	g_string_append (string, remainder);
+	g_string_append (string, _(" (invalid Unicode)"));
+	g_assert (g_utf8_validate (string->str, -1, NULL));
+
+	return g_string_free (string, FALSE);
+}
+
 static gboolean  
 mkisofs_stderr_read (GIOChannel   *source,
 		     GIOCondition  condition,
@@ -672,7 +711,29 @@ mkisofs_stderr_read (GIOChannel   *source,
 			}
 
 			if (strstr (line, "Incorrectly encoded string")) {
-				mkisofs_output_error (mkisofs_output, _("Some files have invalid filenames."));
+				GString *filename;
+				char    *utf8_filename;
+				char    *msg;
+
+				filename = g_string_sized_new (32);
+
+				g_string_assign (filename, line);
+				/* Erase up to the first ( */
+				g_string_erase (filename, 0, strchr (filename->str, '(') - filename->str + 1);
+				/* Truncate down to the last ) */
+				g_string_truncate (filename, strrchr (filename->str, ')') - filename->str - 1);
+				
+				utf8_filename = ncb_make_valid_utf8 (filename->str);
+				msg = g_strdup_printf (_("Some files have invalid filenames: \n%s"), utf8_filename);
+				if (mkisofs_output->debug) {
+					g_print ("mkisofs error: %s\n", msg);
+				}
+
+				mkisofs_output_error (mkisofs_output, msg);
+
+				g_free (msg);
+				g_free (utf8_filename);
+				g_string_free (filename, TRUE);
 			}
 
 			if (strstr (line, "Unknown charset")) {
@@ -1119,6 +1180,23 @@ run_process (struct mkisofs_output    *mkisofs_output,
 	nautilus_burn_progress_set_image_spinning (FALSE);
 }
 
+static char *
+get_image_space_error_message (guint64 iso_size,
+			       guint64 space_available)
+{
+	char   *space_needed_string;
+	char   *message;
+	guint64 space_needed;
+
+	space_needed = iso_size - space_available;
+	space_needed_string = g_strdup_printf ("%" G_GINT64_FORMAT, space_needed / 1048576);
+	message = g_strdup_printf (_("The selected location does not have enough space to store the disc image (%s MiB needed)."),
+				   space_needed_string);
+	g_free (space_needed_string);
+
+	return message;
+}
+
 /**
  * nautilus_burn_make_iso:
  * @filename: name of a file to use for the image
@@ -1302,10 +1380,16 @@ nautilus_burn_make_iso (const char                  *filename,
 		g_warning ("Cannot get free space at %s\n", dirname);
 	} else if (iso_size > size) {
 		if (flags & NAUTILUS_BURN_IMAGE_CREATE_WARN_SPACE) {
+			char *message;
+
+			message = get_image_space_error_message (iso_size,
+								 size);
+
 			mkisofs_output.error = g_error_new (NAUTILUS_BURN_ISO_ERROR, 
 							    NAUTILUS_BURN_ISO_ERROR_GENERAL,
-							    _("The selected location does not have enough space to store the disc image (%ld MiB needed)."),
-							    (unsigned long)iso_size / 1048576);
+							    "%s", message);
+			g_free (message);
+
 			mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 		} else {
 			mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_RETRY;
@@ -1527,6 +1611,7 @@ nautilus_burn_make_iso_from_drive (const char                  *filename,
 	}
 
 	g_ptr_array_free (argv, TRUE);
+	argv = NULL;
 	g_free (stdout_data);
 
 	if (has_audio) {
@@ -1554,10 +1639,16 @@ nautilus_burn_make_iso_from_drive (const char                  *filename,
 	} else if (iso_size > size) {
 		g_free (dirname);
 		if (flags & NAUTILUS_BURN_IMAGE_CREATE_WARN_SPACE) {
+			char *message;
+
+			message = get_image_space_error_message (iso_size,
+								 size);
+
 			mkisofs_output.error = g_error_new (NAUTILUS_BURN_ISO_ERROR, 
 							    NAUTILUS_BURN_ISO_ERROR_GENERAL,
-							    _("The selected location does not have enough space to store the disc image (%ld MiB needed)."),
-							    (unsigned long)iso_size / 1048576);
+							    "%s", message);
+			g_free (message);
+
 			mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 		} else {
 			mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_RETRY;
@@ -1604,7 +1695,10 @@ nautilus_burn_make_iso_from_drive (const char                  *filename,
 	g_free (outfile_arg);
 	g_free (tocfile_arg);
 	g_free (device_arg);
-	g_ptr_array_free (argv, TRUE);
+	if (argv) {
+		g_ptr_array_free (argv, TRUE);
+		argv = NULL;
+	}
 
 	if (mkisofs_output.result == NAUTILUS_BURN_RECORDER_RESULT_ERROR) {
 		if (!mkisofs_output.error)
@@ -1618,45 +1712,6 @@ nautilus_burn_make_iso_from_drive (const char                  *filename,
 	return mkisofs_output.result;
 }
 
-/* Originally eel_make_valid_utf8 */
-static char *
-ncb_make_valid_utf8 (const char *name)
-{
-	GString    *string;
-	const char *remainder, *invalid;
-	int         remaining_bytes, valid_bytes;
-
-	string = NULL;
-	remainder = name;
-	remaining_bytes = strlen (name);
-
-	while (remaining_bytes != 0) {
-		if (g_utf8_validate (remainder, remaining_bytes, &invalid)) {
-			break;
-		}
-		valid_bytes = invalid - remainder;
-
-		if (string == NULL) {
-			string = g_string_sized_new (remaining_bytes);
-		}
-		g_string_append_len (string, remainder, valid_bytes);
-		g_string_append_c (string, '?');
-
-		remaining_bytes -= valid_bytes + 1;
-		remainder = invalid + 1;
-	}
-
-	if (string == NULL) {
-		return g_strdup (name);
-	}
-
-	g_string_append (string, remainder);
-	g_string_append (string, _(" (invalid Unicode)"));
-	g_assert (g_utf8_validate (string->str, -1, NULL));
-
-	return g_string_free (string, FALSE);
-}
-
 /**
  * nautilus_burn_verify_iso:
  * @filename: name of a file to use for the image
