@@ -36,7 +36,6 @@
 #include <string.h>
 #include <unistd.h>
 #include <fcntl.h>
-#include <math.h>
 
 #ifdef __linux__
 #include <linux/cdrom.h>
@@ -83,8 +82,6 @@ int    get_disc_status      (int fd, int *empty, int *is_rewritable, int *is_bla
 #define INVALID_HANDLE (GINT_TO_POINTER(-1))
 #endif
 
-#define CD_ROM_SPEED 176
-
 #define CDS_NO_INFO		0	/* if not implemented */
 #define CDS_AUDIO		100
 #define CDS_DATA_1		101
@@ -208,22 +205,6 @@ close_ioctl_handle (gpointer handle)
 
 #endif
 
-static gboolean
-use_gnome_mount (void)
-{
-#ifndef USE_HAL
-	return FALSE;
-#endif
-
-	char *tmp = g_find_program_in_path ("gnome-mount");
-	if (tmp != NULL) {
-		g_free (tmp);
-		return TRUE;
-	} else {
-		return FALSE;
-	}
-}
-
 #ifdef USE_HAL
 static LibHalContext *
 get_hal_context (void)
@@ -270,11 +251,8 @@ get_device_max_write_speed (char *device)
 {
 	gpointer ioctl_handle;
 	int      fd;
-	int      max_speed;
 	int      read_speed, write_speed;
 
-	max_speed = -1;
-
 	ioctl_handle = open_ioctl_handle (device);
 	if (ioctl_handle == INVALID_HANDLE) {
 		return -1;
@@ -283,9 +261,8 @@ get_device_max_write_speed (char *device)
 	fd = get_ioctl_handle_fd (ioctl_handle);
 	get_read_write_speed (fd, &read_speed, &write_speed);
 	close_ioctl_handle (ioctl_handle);
-	max_speed = (int) floor (write_speed) / CD_ROM_SPEED;
 
-	return max_speed;
+	return write_speed;
 }
 
 /* Utility functions, be careful to have a match with what's use in the
@@ -297,11 +274,8 @@ get_device_max_read_speed (char *device)
 {
 	gpointer ioctl_handle;
 	int      fd;
-	int      max_speed;
 	int      read_speed, write_speed;
 
-	max_speed = -1;
-
 	ioctl_handle = open_ioctl_handle (device);
 	if (ioctl_handle == INVALID_HANDLE) {
 		return -1;
@@ -310,9 +284,8 @@ get_device_max_read_speed (char *device)
 	fd = get_ioctl_handle_fd (ioctl_handle);
 	get_read_write_speed (fd, &read_speed, &write_speed);
 	close_ioctl_handle (ioctl_handle);
-	max_speed = (int)floor (read_speed) / CD_ROM_SPEED;
 
-	return max_speed;
+	return read_speed;
 }
 
 static char *
@@ -942,7 +915,7 @@ nautilus_burn_drive_get_media_type_from_path (const char *device)
  * nautilus_burn_drive_get_media_type:
  * @drive: #NautilusBurnDrive
  *
- * Get the type of the specified drive.
+ * Determine the type of the media in the drive @drive.
  *
  * Return value: See nautilus_burn_drive_get_media_type_from_path() for details.
  **/
@@ -1124,7 +1097,7 @@ nautilus_burn_drive_hal_get_media_type_full (NautilusBurnDrive *drive,
  * @has_data: set to TRUE if media has data
  * @has_audio: set to TRUE if media has audio
  *
- * Get the type of the specified drive.
+ * Determine the type of the media in the drive @drive
  *
  * Return value: See nautilus_burn_drive_get_media_type_from_path() for details.
  **/
@@ -1516,6 +1489,7 @@ unmount_timeout (gpointer data)
 	return FALSE;
 }
 
+#ifndef USE_GNOME_MOUNT
 static const char *umount_known_locations [] = {
 	"/sbin/umount",
 	"/bin/umount",
@@ -1523,28 +1497,28 @@ static const char *umount_known_locations [] = {
 	"/usr/bin/umount",
 	NULL
 };
+#endif
 
 /* Returns the full command */
 static GPtrArray *
 create_command (const char *device)
 {
 	GPtrArray *argv;
+	char      *str;
 
 	argv = g_ptr_array_new ();
 
-	if (use_gnome_mount ()) {
-		char *str;
-
-		str = g_strdup ("gnome-mount");
-		g_ptr_array_add (argv, str);
-		str = g_strdup_printf ("--device=%s", device);
-		g_ptr_array_add (argv, str);
-		str = g_strdup ("--unmount");
-		g_ptr_array_add (argv, str);
-		str = g_strdup ("--no-ui");
-		g_ptr_array_add (argv, str);
-	} else {
-		char *str;
+#ifdef USE_GNOME_MOUNT
+	str = g_strdup (BINDIR "/gnome-mount");
+	g_ptr_array_add (argv, str);
+	str = g_strdup_printf ("--device=%s", device);
+	g_ptr_array_add (argv, str);
+	str = g_strdup ("--unmount");
+	g_ptr_array_add (argv, str);
+	str = g_strdup ("--no-ui");
+	g_ptr_array_add (argv, str);
+#else
+	{
 		int   i;
 
 		str = NULL;
@@ -1564,6 +1538,7 @@ create_command (const char *device)
 		str = g_strdup_printf ("%s", device);
 		g_ptr_array_add (argv, str);
 	}
+#endif
 
 	g_ptr_array_add (argv, NULL);
 
@@ -1573,6 +1548,7 @@ create_command (const char *device)
 static void *
 unmount_thread_start (void *arg)
 {
+	GError      *error;
 	UnmountData *data;
 	gint         exit_status;
 	char        *envp [] = {
@@ -1583,7 +1559,8 @@ unmount_thread_start (void *arg)
 	data = arg;
 
 	data->unmount_ok = TRUE;
-	
+
+	error = NULL;
 	if (g_spawn_sync (NULL,
 			  (char **)data->argv->pdata,
 			  envp,
@@ -1592,7 +1569,7 @@ unmount_thread_start (void *arg)
 			  NULL,
 			  NULL,
 			  &exit_status,
-			  NULL)) {
+			  &error)) {
 		if (exit_status == 0) {
 			data->unmount_ok = TRUE;
 		} else {
@@ -1603,6 +1580,10 @@ unmount_thread_start (void *arg)
 		sleep (2);
 	} else {
 		/* spawn failure */
+		if (error) {
+			g_warning ("Unable to unmount: %s", error->message);
+			g_error_free (error);
+		}
 		data->unmount_ok = FALSE;
 	}
 
@@ -1768,14 +1749,15 @@ nautilus_burn_drive_eject (NautilusBurnDrive *drive)
 
 	g_return_val_if_fail (drive != NULL, FALSE);
 
-	if (drive->device == NULL)
+	if (drive->device == NULL) {
 		return FALSE;
-	
-	if (use_gnome_mount ()) {
-		cmd = g_strdup_printf ("gnome-mount --eject --no-ui --device=%s", drive->device);
-	} else {
-		cmd = g_strdup_printf ("eject %s", drive->device);
 	}
+	
+#ifdef USE_GNOME_MOUNT
+	cmd = g_strdup_printf ("gnome-mount --eject --no-ui --device=%s", drive->device);
+#else
+	cmd = g_strdup_printf ("eject %s", drive->device);
+#endif
 
 	res = g_spawn_command_line_sync (cmd, NULL, NULL, NULL, NULL);
 	g_free (cmd);
@@ -1943,8 +1925,6 @@ hal_parse_write_speeds (char **strlist)
 			g_free (write_speeds);
 			return NULL;
 		}
-
-		write_speeds[i] /= CD_ROM_SPEED;
 	}
 
 	return write_speeds;
@@ -1953,7 +1933,6 @@ hal_parse_write_speeds (char **strlist)
 #define LIBHAL_PROP_EXTRACT_BEGIN if (FALSE)
 #define LIBHAL_PROP_EXTRACT_END ;
 #define LIBHAL_PROP_EXTRACT_INT(_property_, _where_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_INT32) _where_ = libhal_psi_get_int (&it)
-#define LIBHAL_PROP_EXTRACT_INT_DIV(_property_, _where_, _div_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_INT32) _where_ = libhal_psi_get_int (&it) / _div_
 #define LIBHAL_PROP_EXTRACT_STRING(_property_, _where_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_STRING) _where_ = (libhal_psi_get_string (&it) != NULL && strlen (libhal_psi_get_string (&it)) > 0) ? strdup (libhal_psi_get_string (&it)) : NULL
 #define LIBHAL_PROP_EXTRACT_BOOL(_property_, _where_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_BOOLEAN) _where_ = libhal_psi_get_bool (&it)
 #define LIBHAL_PROP_EXTRACT_BOOL_BITFIELD(_property_, _where_, _field_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_BOOLEAN) _where_ |= libhal_psi_get_bool (&it) ? _field_ : 0
@@ -1997,8 +1976,8 @@ hal_drive_from_udi (LibHalContext *ctx,
 		LIBHAL_PROP_EXTRACT_STRING  ("block.device",               drive->device);
 		LIBHAL_PROP_EXTRACT_STRING  ("storage.model",              drive->display_name);
 
-		LIBHAL_PROP_EXTRACT_INT_DIV ("storage.cdrom.read_speed",   drive->max_speed_read,  CD_ROM_SPEED);
-		LIBHAL_PROP_EXTRACT_INT_DIV ("storage.cdrom.write_speed",  drive->max_speed_write, CD_ROM_SPEED);
+		LIBHAL_PROP_EXTRACT_INT ("storage.cdrom.read_speed",   drive->max_speed_read);
+		LIBHAL_PROP_EXTRACT_INT ("storage.cdrom.write_speed",  drive->max_speed_write);
 		LIBHAL_PROP_EXTRACT_STRLIST ("storage.cdrom.write_speeds", write_speeds);
 
 		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.cdr",        drive->type, NAUTILUS_BURN_DRIVE_TYPE_CD_RECORDER);
@@ -2376,7 +2355,7 @@ add_linux_cd_recorder (GList             *drives,
 		drive->priv->protocol = NAUTILUS_BURN_DRIVE_PROTOCOL_SCSI;
 		if (!get_cd_scsi_id (drive_s->device, &bus, &id, &lun)) {
 			g_free (drive->display_name);
-			g_free (drive);
+			nautilus_burn_drive_unref (drive);
 			return drives;
 		}
 		drive->cdrecord_id = g_strdup_printf ("%d,%d,%d",
