@@ -30,6 +30,7 @@
 
 #include <glib-object.h>
 #include <glib.h>
+#include <math.h>
 
 #include "nautilus-burn-features.h"
 
@@ -102,6 +103,8 @@ struct NautilusBurnDriveClass {
 };
 
 #define NAUTILUS_BURN_DRIVE_SIZE_TO_TIME(size) (size > 0 ? (int) (((size / 1024 / 1024) - 1) * 48 / 7): 0)
+#define NAUTILUS_BURN_DRIVE_CD_SPEED(speed) ((int) floor (speed * 1024 / 153600))
+#define NAUTILUS_BURN_DRIVE_DVD_SPEED(speed) ((int) floor (speed * 1024 / 1385000))
 
 GType                 nautilus_burn_drive_get_type                 (void);
 
