@@ -30,7 +30,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
-#include <math.h>
 #include <fcntl.h>
 #include <sys/wait.h>
 #include <sys/types.h>
@@ -73,6 +72,7 @@ struct _NautilusBurnProcess {
 
 	time_t                    start_time;
 	gint64                    start_num;
+	GList                    *rates;
 
 	gboolean                  changed_text;
 	gboolean                  expect_process_to_die;
@@ -170,15 +170,21 @@ nautilus_burn_process_new ()
 static void
 nautilus_burn_process_free (NautilusBurnProcess *process)
 {
-	if (! process)
+	if (! process) {
 		return;
+	}
 
 	g_free (process->last_error);
 
-	if (process->line)
+	if (process->line) {
 		g_string_free (process->line, TRUE);
-	if (process->line_stderr)
+	}
+
+	if (process->line_stderr) {
 		g_string_free (process->line_stderr, TRUE);
+	}
+
+	g_list_free (process->rates);
 
 	g_free (process->funcs);
 	g_free (process);
@@ -329,8 +335,9 @@ compute_time_remaining (gint64 bytes,
 {
 	long secs;
 
-	if (bytes_per_sec <= 1)
+	if (bytes_per_sec <= 1) {
 		return -1;
+	}
 
 	secs = bytes / bytes_per_sec;
 
@@ -413,6 +420,34 @@ growisofs_blank_stdout_line (NautilusBurnProcess   *process,
 	return TRUE;
 }
 
+static gdouble
+get_average_rate (GList  **rates,
+		  gdouble  rate)
+{
+	const unsigned int max_num = 16;
+	const unsigned int scale  = 1000;
+	gdouble            average;
+	gint32             int_rate;
+	GList             *l;
+
+	if (g_list_length (*rates) > max_num) {
+		*rates = g_list_delete_link (*rates, *rates);
+	}
+	int_rate = (gint32)ceil (scale * rate);
+
+	*rates = g_list_append (*rates, GINT_TO_POINTER (int_rate));
+
+	average = 0;
+	for (l = *rates; l != NULL; l = l->next) {
+		gdouble r = (gdouble)GPOINTER_TO_INT (l->data) / scale;
+		average += r;
+	}
+
+	average /= g_list_length (*rates);
+
+	return average;
+}
+
 static gboolean
 cdrecord_stdout_line (NautilusBurnProcess   *process,
 		      const char            *line,
@@ -437,10 +472,11 @@ cdrecord_stdout_line (NautilusBurnProcess   *process,
 				
 		process->result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 				
-		if (recorder->priv->tracks_total_bytes > 0)
+		if (recorder->priv->tracks_total_bytes > 0) {
 			total = recorder->priv->tracks_total_bytes;
-		else
+		} else {
 			total = mb_total * 1048576;
+		}
 
 		if (track > recorder->priv->current_track_num) {
 			recorder->priv->current_track_num = track;
@@ -450,7 +486,16 @@ cdrecord_stdout_line (NautilusBurnProcess   *process,
 		this_remain = (mb_total - mb_written) * 1048576;
 		bytes = (total - recorder->priv->current_track_end_pos) + this_remain;
 
-		secs = compute_time_remaining (bytes, (double)speed * CDR_SPEED);
+		secs = -1;
+		if (speed > 0) {
+			gdouble ave_rate;
+			gdouble rate;
+
+			rate = (double)speed * CDR_SPEED;
+
+			ave_rate = get_average_rate (&process->rates, rate);
+			secs = compute_time_remaining (bytes, ave_rate);
+		}
 
 		if (!process->changed_text) {
 			g_signal_emit (recorder,
@@ -469,9 +514,10 @@ cdrecord_stdout_line (NautilusBurnProcess   *process,
 		g_signal_emit (recorder,
 			       nautilus_burn_recorder_table_signals [PROGRESS_CHANGED], 0,
 			       percent, secs);
-	} else if (sscanf (line, "Total size:  %d MB ", &mb_total)) {
+	} else if (sscanf (line, "Track %*d: %*s %d MB ", &mb_total)) {
+		/* %s can be either "data" or "audio" */
 		if (mb_total > 0) {
-			recorder->priv->tracks_total_bytes = mb_total * 1048576;
+			recorder->priv->tracks_total_bytes += mb_total * 1048576;
 		}
 	} else if (g_str_has_prefix (line, "Re-load disk and hit <CR>") ||
 		   g_str_has_prefix (line, "send SIGUSR1 to continue")) {
@@ -498,7 +544,7 @@ cdrecord_stdout_line (NautilusBurnProcess   *process,
 	} else if (g_str_has_prefix (line, "Fixating time:")) {
 		g_signal_emit (recorder,
 			       nautilus_burn_recorder_table_signals [PROGRESS_CHANGED], 0,
-			       1.0, -1);
+			       1.0, (long)-1);
 		process->result = NAUTILUS_BURN_RECORDER_RESULT_FINISHED;
 	} else if (g_str_has_prefix (line, "Last chance to quit, ")) {
 		process->dangerous = TRUE;
@@ -581,7 +627,7 @@ dvdrw_format_stderr_line (NautilusBurnProcess   *process,
 
 			g_signal_emit (recorder,
 				       nautilus_burn_recorder_table_signals [PROGRESS_CHANGED], 0,
-				       percent, -1);
+				       percent, (long)-1);
 		}
 
 	} else {
@@ -719,51 +765,6 @@ cdrecord_stderr_line (NautilusBurnProcess   *process,
 	return TRUE;
 }
 
-static gboolean
-media_type_matches (NautilusBurnMediaType type,
-		    gboolean              is_blank,
-		    gboolean              was_unmounted)
-{
-	if (was_unmounted) {
-		switch (type) {
-		case NAUTILUS_BURN_MEDIA_TYPE_CDRW:
-		case NAUTILUS_BURN_MEDIA_TYPE_DVDRW:
-		case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW:
-			return TRUE;
-		default:
-			return FALSE;
-		}
-	}
-
-	switch (type) {
-	case NAUTILUS_BURN_MEDIA_TYPE_ERROR:
-	case NAUTILUS_BURN_MEDIA_TYPE_BUSY:
-		return FALSE;
-	case NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN:
-		return TRUE;
-	case NAUTILUS_BURN_MEDIA_TYPE_CD:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVD:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVD_RAM:
-		return FALSE;
-	case NAUTILUS_BURN_MEDIA_TYPE_CDR:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVDR:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R_DL:
-		if (is_blank)
-			return TRUE;
-		else
-			return FALSE;
-	case NAUTILUS_BURN_MEDIA_TYPE_CDRW:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVDRW:
-	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW:
-		return TRUE;
-	default:
-		break;
-	}
-
-	return FALSE;
-}
-
 static NautilusBurnMediaType
 nautilus_burn_recorder_wait_for_insertion (NautilusBurnRecorder *recorder,
 					   NautilusBurnDrive    *drive,
@@ -771,68 +772,67 @@ nautilus_burn_recorder_wait_for_insertion (NautilusBurnRecorder *recorder,
 					   gboolean             *blank)
 {
 	NautilusBurnMediaType type;
+	gint64                media_size;
 	gboolean              reload;
-	gboolean              last_was_unmount;
 	gboolean              is_rewritable;
+	gboolean              can_write;
 	gboolean              is_blank;
 	gboolean              has_data;
 	gboolean              has_audio;
 
-	last_was_unmount = FALSE;
 	reload = FALSE;
-	type = nautilus_burn_drive_get_media_type_full (drive, &is_rewritable, &is_blank, &has_data, &has_audio);
 
-	if (can_rewrite)
-		*can_rewrite = is_rewritable;
+ again:
 
-	if (blank)
-		*blank = is_blank;
-
-	last_was_unmount = FALSE;
-	while (!media_type_matches (type, is_blank, last_was_unmount)) {
-		gboolean res, busy_cd;
-
-		busy_cd = (type == NAUTILUS_BURN_MEDIA_TYPE_BUSY);
-
-		if (busy_cd && !last_was_unmount) {
-			/* umount the disc if we find it, and try it again */
-			if (nautilus_burn_drive_unmount (drive)) {
-				last_was_unmount = TRUE;
-				goto retry;
-			}
+	/* if drive is mounted then unmount before checking anything */
+	if (nautilus_burn_drive_is_mounted (drive)) {
+		gboolean res;
+		res = nautilus_burn_drive_unmount (drive);
+		if (! res) {
+			g_warning ("Couldn't unmount volume in drive: %s", drive->device);
 		}
+	}
 
+	media_size = nautilus_burn_drive_get_media_size (drive);
+	type = nautilus_burn_drive_get_media_type_full (drive, &is_rewritable, &is_blank, &has_data, &has_audio);
+	can_write = nautilus_burn_drive_media_type_is_writable (type, is_blank);
+
+	if (can_rewrite) {
+		*can_rewrite = is_rewritable;
+	}
+
+	if (blank) {
+		*blank = is_blank;
+	}
+
+	if (! can_write) {
+		gboolean is_mounted;
+		int      res;
+
+		reload = (media_size > 0);
 		if (type == NAUTILUS_BURN_MEDIA_TYPE_ERROR) {
 			reload = FALSE;
 		}
 
+		/* check one more time */
+		is_mounted = nautilus_burn_drive_is_mounted (drive);
+
 		g_signal_emit (recorder,
 			       nautilus_burn_recorder_table_signals [INSERT_MEDIA_REQUEST],
-			       0, reload, recorder->priv->can_rewrite,
-			       busy_cd, &res);
+			       0,
+			       reload,
+			       recorder->priv->can_rewrite,
+			       is_mounted,
+			       &res);
 
 		if (res == FALSE) {
 			return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
 		}
 
-		last_was_unmount = FALSE;
-
-	retry:
-		type = nautilus_burn_drive_get_media_type_full (drive, &is_rewritable, &is_blank, &has_data, &has_audio);
-
-		if (can_rewrite)
-			*can_rewrite = is_rewritable;
-
-		if (blank)
-			*blank = is_blank;
-
-		reload = FALSE;
-		if (type == NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN) {
-			reload = TRUE;
-		}
+		goto again;
 	}
 
-	if (!is_blank) {
+	if (! is_blank) {
 		int res;
 
 		res = 0;
@@ -843,7 +843,7 @@ nautilus_burn_recorder_wait_for_insertion (NautilusBurnRecorder *recorder,
 			       0, &res);
 
 		if (res == NAUTILUS_BURN_RECORDER_RESPONSE_RETRY) {
-			goto retry;
+			goto again;
 		} else if (res == NAUTILUS_BURN_RECORDER_RESPONSE_CANCEL) {
 			type = NAUTILUS_BURN_MEDIA_TYPE_ERROR;
 		}
@@ -1231,7 +1231,7 @@ nautilus_burn_recorder_run_process (NautilusBurnRecorder       *recorder,
 		       media_type);
 	g_signal_emit (G_OBJECT (recorder),
 		       nautilus_burn_recorder_table_signals [PROGRESS_CHANGED], 0,
-		       0.0, -1);
+		       0.0, (long)-1);
 	g_signal_emit (G_OBJECT (recorder),
 		       nautilus_burn_recorder_table_signals [ANIMATION_CHANGED], 0,
 		       TRUE);
@@ -1317,7 +1317,8 @@ nautilus_burn_recorder_write_growisofs (NautilusBurnRecorder          *recorder,
 					GError                       **error)
 {
 	GPtrArray                 *argv;
-	char                      *speed_str, *dev_str;
+	char                      *speed_str = NULL;
+	char			  *dev_str;
 	NautilusBurnRecorderTrack *t;
 	int                        result;
 
@@ -1335,8 +1336,10 @@ nautilus_burn_recorder_write_growisofs (NautilusBurnRecorder          *recorder,
 	argv = g_ptr_array_new ();
 
 	g_ptr_array_add (argv, "growisofs");
-	speed_str = g_strdup_printf ("-speed=%d", speed);
 	if (speed != 0) {
+		speed = NAUTILUS_BURN_DRIVE_DVD_SPEED (speed);
+
+		speed_str = g_strdup_printf ("-speed=%d", speed);
 		g_ptr_array_add (argv, speed_str);
 	}
 
@@ -1436,7 +1439,7 @@ cdrdao_stderr_line (NautilusBurnProcess   *process,
 			       NAUTILUS_BURN_RECORDER_MEDIA_CD);
 		g_signal_emit (recorder,
 			       nautilus_burn_recorder_table_signals [PROGRESS_CHANGED], 0,
-			       0.0, -1);
+			       0.0, (long)-1);
 
 	}
 
@@ -1445,7 +1448,7 @@ cdrdao_stderr_line (NautilusBurnProcess   *process,
 		process->result = NAUTILUS_BURN_RECORDER_RESULT_FINISHED;
 		g_signal_emit (recorder,
 			       nautilus_burn_recorder_table_signals [PROGRESS_CHANGED], 0,
-			       1.0, -1);
+			       1.0, (long)-1);
 
 	}
 
@@ -1494,6 +1497,7 @@ nautilus_burn_recorder_write_cdrecord (NautilusBurnRecorder          *recorder,
 			return res;
 	}
 
+	speed = NAUTILUS_BURN_DRIVE_CD_SPEED (speed);
 
 	if (track->type ==  NAUTILUS_BURN_RECORDER_TRACK_TYPE_CUE) {
 		argv = g_ptr_array_new ();
