@@ -46,6 +46,7 @@
 #include "nautilus-burn-drive-selection.h"
 #include "nautilus-cd-burner.h"
 #include "make-iso.h"
+#include "nautilus-file-operations-progress-icons.h"
 
 #define MAX_ISO_NAME_LEN  32
 #define RESPONSE_OVERBURN 1
@@ -97,6 +98,8 @@ static int                      cancel   = CANCEL_NONE;
 static GList                   *image_list;
 static GList                   *files_for_cleanup = NULL;
 static NautilusBurnerWriteData *write_data = NULL;
+static int                      progress_jar_position = 0;
+static GdkPixbuf               *empty_jar_pixbuf, *full_jar_pixbuf;
 
 static int write_disc (NautilusBurnerWriteData *data);
 
@@ -219,6 +222,33 @@ nautilus_burn_progress_get_window (void)
 	return GTK_WINDOW (glade_xml_get_widget (xml, "progress_window"));
 }
 
+static void
+nautilus_burn_progress_update_icon (double fraction)
+{
+	GdkPixbuf *pixbuf;
+	GtkWindow *progress;
+	int        position;
+
+	position = gdk_pixbuf_get_height (empty_jar_pixbuf) * (1 - fraction);
+
+	if (position == progress_jar_position) {
+		return;
+	}
+
+	progress_jar_position = position;
+	
+	pixbuf = gdk_pixbuf_copy (empty_jar_pixbuf);
+	gdk_pixbuf_copy_area (full_jar_pixbuf,
+			      0, position,
+			      gdk_pixbuf_get_width (pixbuf), gdk_pixbuf_get_height (pixbuf) - position,
+			      pixbuf,
+			      0, position);
+
+	progress = nautilus_burn_progress_get_window ();
+	gtk_window_set_icon (GTK_WINDOW (progress), pixbuf);
+	g_object_unref (pixbuf);
+}
+
 void 
 nautilus_burn_progress_set_fraction (double fraction)
 {
@@ -227,6 +257,7 @@ nautilus_burn_progress_set_fraction (double fraction)
 	progress_bar = glade_xml_get_widget (xml, "cd_progress");
 
 	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), fraction);
+	nautilus_burn_progress_update_icon (fraction);
 }
 
 /* Adapted from totem_time_to_string_text */
@@ -381,6 +412,10 @@ nautilus_burn_progress_image_setup (void)
 
 	/* Set the first image */
 	nautilus_burn_progress_set_image (NULL);
+
+	/* Load the jar pixbufs */
+	empty_jar_pixbuf = gdk_pixbuf_new_from_inline (-1, progress_jar_empty_icon, FALSE, NULL);
+	full_jar_pixbuf = gdk_pixbuf_new_from_inline (-1, progress_jar_full_icon, FALSE, NULL);
 }
 
 static void
@@ -905,14 +940,17 @@ setup_cancel_button (void)
 	GtkWidget *close_button;
 	GtkWidget *cancel_button;
 	GtkWidget *write_another_button;
+	GtkWidget *eject_button;
 
 	cancel_button = glade_xml_get_widget (xml, "cancel_button");
 	close_button = glade_xml_get_widget (xml, "close_button");
 	write_another_button = glade_xml_get_widget (xml, "write_another_button");
+	eject_button = glade_xml_get_widget (xml, "eject_button");
 
 	gtk_widget_show (cancel_button);
 	gtk_widget_hide (close_button);
 	gtk_widget_hide (write_another_button);
+	gtk_widget_hide (eject_button);
 
 	/* FIXME: disconnect signal handlers? */
 }
@@ -1261,6 +1299,11 @@ write_disc (NautilusBurnerWriteData *data)
 			       insert_media_info,
 			       (GClosureNotify) g_free, 0);
 
+	/* Set window icon */
+	gtk_window_set_icon (nautilus_burn_progress_get_window (), empty_jar_pixbuf);
+
+	/* Set progress jar position */
+	progress_jar_position = gdk_pixbuf_get_height (empty_jar_pixbuf);
 
 	res = nautilus_burn_recorder_write_tracks (recorder,
 						   data->drive,
@@ -1752,65 +1795,138 @@ get_speed_selection (GtkWidget *combobox)
 }
 
 static void
-refresh_dialog (GladeXML *xml)
+update_speeds_combobox (NautilusBurnDrive *drive,
+			GtkWidget	  *combobox)
 {
-	GtkWidget               *combobox;
+	const int		*write_speeds;
+	int			 i;
+	GtkTreeModel		*model;
+	GtkTreeIter		 iter;
+	NautilusBurnMediaType	 type;
 	GConfClient             *gc;
-	char                    *name;
-	int                      i;
+	gboolean		 is_rewritable;
+	gboolean		 is_blank;
 	int                      default_speed;
-	int                      default_speed_index;
-	NautilusBurnDrive       *drive;
-	GtkTreeModel            *model;
-	GtkTreeIter              iter;
+	int                      default_speed_index = 0;
 
-	default_speed_index = 0;
-
-	/* Find active recorder: */
-	drive = lookup_current_recorder (xml);
-
-	/* add speed items: */
-	combobox = glade_xml_get_widget (xml, "speed_combobox");
-	model = gtk_combo_box_get_model (GTK_COMBO_BOX (combobox));
-	gtk_list_store_clear (GTK_LIST_STORE (model));
 
 	gc = gconf_client_get_default ();
 	default_speed = gconf_client_get_int (gc, "/apps/nautilus-cd-burner/default_speed", NULL);
 	g_object_unref (G_OBJECT (gc));
 
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX (combobox));
+
+	gtk_list_store_clear (GTK_LIST_STORE (model));
+
 	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
 	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 			    0, _("Maximum possible"),
 			    1, 0,
 			    -1);
 
-	if (drive) {
-		const int *write_speeds;
+	write_speeds = nautilus_burn_drive_get_write_speeds (drive);
+	type = nautilus_burn_drive_get_media_type_full (drive,
+							&is_rewritable,
+							&is_blank,
+							NULL, NULL);
 
-		write_speeds = nautilus_burn_drive_get_write_speeds (drive);
+	for (i = 0; write_speeds[i] > 0; i++) {
+		char *name;
 
-		for (i = 0; write_speeds[i] > 0; i++) {
+		name = NULL;
 
-			if (write_speeds[i] == default_speed) {
-				default_speed_index = i + 1;
-			}
-
-			name = g_strdup_printf ("%d \303\227", write_speeds[i]);
-			gtk_list_store_append (GTK_LIST_STORE (model), &iter);
-			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-					    0, name,
-					    1, write_speeds [i],
-					    -1);
-			g_free (name);
+		if (write_speeds[i] == default_speed) {
+			default_speed_index = i + 1;
 		}
 
-		/* Disable speed if no items in list */
-		gtk_widget_set_sensitive (combobox, i > 0);
+		if (is_rewritable || is_blank) {
+			switch (type) {
+			case NAUTILUS_BURN_MEDIA_TYPE_CDR:
+			case NAUTILUS_BURN_MEDIA_TYPE_CDRW:
+				name = g_strdup_printf ("%d\303\227",
+							NAUTILUS_BURN_DRIVE_CD_SPEED (write_speeds[i]));
+				break;
+			case NAUTILUS_BURN_MEDIA_TYPE_DVDR:
+			case NAUTILUS_BURN_MEDIA_TYPE_DVDRW:
+			case NAUTILUS_BURN_MEDIA_TYPE_DVD_RAM:
+			case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R:
+			case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW:
+			case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R_DL:
+				name = g_strdup_printf ("%d\303\227",
+							NAUTILUS_BURN_DRIVE_DVD_SPEED (write_speeds[i]));
+				break;
+			default:
+				break;
+			}
+
+		} else {
+			name = g_strdup_printf ("CD %d\303\227, DVD %d\303\227",
+						NAUTILUS_BURN_DRIVE_CD_SPEED (write_speeds[i]),
+						NAUTILUS_BURN_DRIVE_DVD_SPEED (write_speeds[i]));
+		}
+
+		/* If we can't get the media type see if the drive is capable DVD */
+		if (name == NULL) {
+			gboolean is_dvd;
+
+			is_dvd = (drive->type & NAUTILUS_BURN_DRIVE_TYPE_DVD_RW_RECORDER)
+				|| (drive->type & NAUTILUS_BURN_DRIVE_TYPE_DVD_RAM_RECORDER);
+			if (is_dvd) {
+				name = g_strdup_printf ("CD %d\303\227, DVD %d\303\227",
+							NAUTILUS_BURN_DRIVE_CD_SPEED (write_speeds[i]),
+							NAUTILUS_BURN_DRIVE_DVD_SPEED (write_speeds[i]));
+			} else {
+				name = g_strdup_printf ("%d\303\227",
+							NAUTILUS_BURN_DRIVE_CD_SPEED (write_speeds[i]));
+			}
+		}
+
+		gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+		gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+				    0, name,
+				    1, write_speeds [i],
+				    -1);
+		g_free (name);
 	}
 
+	/* Disable speed if no items in list */
+	gtk_widget_set_sensitive (combobox, i > 0);
+
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combobox), default_speed_index);
+}
+
+static void
+dialog_media_added_cb (NautilusBurnDrive *drive,
+		       gpointer		  data)
+{
+	update_speeds_combobox (drive, GTK_WIDGET (data));
+}
+
+static void
+refresh_dialog (GladeXML *xml)
+{
+	GtkWidget               *combobox;
+	NautilusBurnDrive       *drive;
+	GtkTreeModel            *model;
+
+	/* Find active recorder: */
+	drive = lookup_current_recorder (xml);
+
+	/* add speed items: */
+	combobox = glade_xml_get_widget (xml, "speed_combobox");
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX (combobox));
+
+	g_object_set (drive, "enable-monitor", TRUE, NULL);
+	g_signal_connect (drive, "media-added", G_CALLBACK (dialog_media_added_cb), combobox);
 
 	if (drive) {
+		if (drive->type == NAUTILUS_BURN_DRIVE_TYPE_FILE) {
+			gtk_list_store_clear (GTK_LIST_STORE (model));
+			gtk_widget_set_sensitive (combobox, FALSE);
+		} else {
+			update_speeds_combobox (drive, combobox);
+		}
+
 		nautilus_burn_drive_unref (drive);
 	}
 }
@@ -1983,18 +2099,18 @@ ask_cancel (void)
 	return (res == GTK_RESPONSE_YES);
 }
 
-static gboolean
-handle_cancel (void)
+static void
+do_cancel (void)
 {
 	gboolean cancelled;
 
 	if (cancel == CANCEL_NONE) {
-		return FALSE;
+		return;
 	}
 
 	if (cancel == CANCEL_MAKE_ISO) {
 		nautilus_burn_make_iso_cancel ();
-		return TRUE;
+		return;
 	}
 
 	cancelled = nautilus_burn_recorder_cancel (recorder, TRUE);
@@ -2003,26 +2119,23 @@ handle_cancel (void)
 		if (ask_cancel () == TRUE) {
 			/* do the dangerous cancel */
 			nautilus_burn_recorder_cancel (recorder, FALSE);
-		} else {
-			return TRUE;
 		}
 	}
-
-	return TRUE;
 }
 
 static gboolean
 handle_delete_event (GtkWidget	     *widget,
 		     GdkEventAny     *event)
 {
-	return handle_cancel ();
+	do_cancel ();
+	return TRUE;
 }
 
 static void
 cancel_clicked (GtkButton *button,
 		gpointer   data)
 {
-	handle_cancel ();
+	do_cancel ();
 }
 
 static void
@@ -2478,23 +2591,28 @@ main (int argc, char *argv[])
 	static char              *source_device;
 	static char              *source_iso;
 	static char              *source_cue;
-        struct poptOption         popt_options []  = {
-		{ "source-device", 0, POPT_ARG_STRING, &source_device,
-		  0, N_("Use CD/DVD device as source instead of burn:///"), NULL },
-		{ "source-iso", 0, POPT_ARG_STRING, &source_iso,
-		  0, N_("Use ISO image as source instead of burn:///"), NULL },
-		{ "source-cue", 0, POPT_ARG_STRING, &source_cue,
-		  0, N_("Use CUE/TOC file as source instead of burn:///"), NULL },
-                POPT_TABLEEND
+	GOptionContext           *context;
+	static const GOptionEntry options []  = {
+		{ "source-device", 0, 0, G_OPTION_ARG_FILENAME, &source_device,
+		  N_("Use CD/DVD device as source instead of burn:///"), N_("DEVICE") },
+		{ "source-iso", 0, 0, G_OPTION_ARG_FILENAME, &source_iso,
+		  N_("Use ISO image as source instead of burn:///"), N_("FILENAME") },
+		{ "source-cue", 0, 0, G_OPTION_ARG_FILENAME, &source_cue,
+		  N_("Use CUE/TOC file as source instead of burn:///"), N_("FILENAME") },
+		{ NULL}
 	};
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
+	context = g_option_context_new (NULL);
+
+	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+
 	gnome_program_init ("nautilus-cd-burner", VERSION,
 			    LIBGNOMEUI_MODULE, argc, argv,
-			    GNOME_PARAM_POPT_TABLE, popt_options,
+			    GNOME_PARAM_GOPTION_CONTEXT, context,
 			    GNOME_PARAM_APP_DATADIR, SHAREDIR,
 			    GNOME_PARAM_NONE);
     
